Contribution getting started — tox 2.9.0rc2.dev54 documentation

# Contribution getting started

Contributions are highly welcomed and appreciated. Every little help counts, so do not hesitate! If you like tox, also share some love on Twitter or in your blog posts.

Contribution links

- Contribution getting started
  - Feature requests and feedback
  - Report bugs
  - Fix bugs
  - Implement features
  - Write documentation
  - Preparing Pull Requests
  - Submitting plugins to tox-dev

## Feature requests and feedback

We’d also like to hear about your propositions and suggestions. Feel free to submit them as issues and:

- Explain in detail how they should work.
- Keep the scope as narrow as possible. This will make it easier to implement.

## Report bugs

Report bugs for tox in the issue tracker.

If you are reporting a bug, please include:

- Your operating system name and version.
- Any details about your local setup that might be helpful in troubleshooting, specifically the Python interpreter version, installed libraries, and tox version.
- Detailed steps to reproduce the bug, or - even better, a n xfaling test reproduces the bug

If you can write a demonstration test that currently fails but should pass (xfail), that is a very useful commit to make as well, even if you cannot fix the bug itself (e.g. something like this in test_config

## Fix bugs

Look through the GitHub issues for bugs. Here is a filter you can use: https://github.com/tox-dev/tox/labels/bug

Don’t forget to check the issue trackers of your favourite plugins, too!

## Implement features

Look through the GitHub issues for enhancements. Here is a filter you can use: https://github.com/tox-dev/tox/labels/enhancement

## Write documentation

tox could always use more documentation. What exactly is needed?

- More complementary documentation. Have you perhaps found something unclear?
- Docstrings. There can never be too many of them.
- Blog posts, articles and such – they’re all very appreciated.

You can also edit documentation files directly in the GitHub web interface, without using a local copy. This can be convenient for small fixes.

Note

Build the documentation locally with the following command:

    $ tox -e docs

The built documentation should be available in the `doc/_build/`.

## Preparing Pull Requests

### Short version

1. Fork the repository

2. Make your changes.

3. open a pull request targeting the `master` branch.

4. Follow PEP-8. There’s a `tox` command to help fixing it: `tox -e fix-lint`. You can also add a pre commit hook to your local clone to run the style checks and fixes (see hint after running `tox -e fix-lint`)

5. Tests for tox are (obvioulsy) run using `tox`:

       tox -e fix-lint,py27,py36

   The test environments above are usually enough to cover most cases locally.

6. Consider the checklist in the pull request form

### Long version

What is a “pull request”? It informs the project’s core developers about the changes you want to review and merge. Pull requests are stored on GitHub servers. Once you send a pull request, we can discuss its potential modifications and even add more commits to it later on. There’s an excellent tutorial on how Pull Requests work in the GitHub Help Center.

Here is a simple overview, with tox-specific bits:

1. Fork the tox GitHub repository. It’s fine to use `tox` as your fork repository name because it will live under your user.

2. Clone your fork locally using git and create a branch:

       $ git clone git@github.com:YOUR_GITHUB_USERNAME/tox.git
       $ cd tox
       # now, to fix a bug create your own branch off "master":

           $ git checkout -b your-bugfix-branch-name master

       # or to instead add a feature create your own branch off "features":

           $ git checkout -b your-feature-branch-name features

   If you need some help with Git, follow this quick start guide: https://git.wiki.kernel.org/index.php/QuickStart

3. Install tox

   Of course tox is used to run all the tests of itself:

       $ cd </path/to/your/tox/clone>
       $ pip install [-e] .

4. Run all the tests

   You need to have Python 2.7 and 3.6 available in your system. Now running tests is as simple as issuing this command:

       $ tox -e fix-lint,py27,py36

   This command will run tests via the “tox” tool against Python 2.7 and 3.6 and also perform style checks with some automatic fixes.

5. You can now edit your local working copy. Please follow PEP-8.

   You can now make the changes you want and run the tests again as necessary.

   > $ tox -e py27 – –pdb

   Or to only run tests in a particular test module on Python 3.6:

       $ tox -e py36 -- testing/test_config.py

   You can also use the dev environment:

   > $ tox -e dev

   To get information about all environements, type:

   $ tox -av

6. Commit and push once your tests pass and you are happy with your change(s):

       $ git commit -a -m "<commit message>"
       $ git push -u

7. submit a pull request through the GitHub website and and consider the checklist in the pull request form:

       head-fork: YOUR_GITHUB_USERNAME/tox
       compare: your-branch-name

       base-fork: tox-dev/tox
       base: master

## Submitting plugins to tox-dev

tox development of the core, some plugins and support code happens in repositories living under the `tox-dev` organisation:

- tox-dev on GitHub

All tox-dev team members have write access to all contained repositories. tox core and plugins are generally developed using pull requests to respective repositories.

The objectives of the `tox-dev` organisation are:

- Having a central location for popular tox plugins
- Sharing some of the maintenance responsibility (in case a maintainer no longer wishes to maintain a plugin)

You can submit your plugin by opening an issue requesting to add you as a member of tox-dev to be able to integrate the plugin. As a member of the or you can then transfer the plugin yourself.

The plugin must have the following:

- PyPI presence with a `setup.py` that contains a license, `tox-` prefixed name, version number, authors, short and long description.
- a `tox.ini` for running tests using tox.
- a `README.txt` describing how to use the plugin and on which platforms it runs.
- a `LICENSE.txt` file or equivalent containing the licensing information, with matching info in `setup.py`.
- an issue tracker for bug reports and enhancement requests.
- a changelog

If no contributor strongly objects, the repository can then be transferred to the `tox-dev` organisation. For details see about repository transfers

Memebrs of the tox organization have write access to all projects. We recommend that each plugin has at least three people who have the right to release to PyPI.

Repository owners can rest assured that no `tox-dev` administrator will ever make releases of your repository or take ownership in any way, except in rare cases where someone becomes unresponsive after months of contact attempts. As stated, the objective is to share maintenance and avoid “plugin-abandon”.